@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2014 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -40,6 +40,7 @@
 
 #include "V3Global.h"
 #include "V3File.h"
+#include "V3Os.h"
 #include "V3PreShell.h"
 #include "V3Ast.h"
 
@@ -259,19 +260,11 @@ bool V3File::checkTimes(const string& filename, const string& cmdline) {
     return dependImp.checkTimes(filename, cmdline);
 }
 
-void V3File::createDir(const string& dirname) {
-#ifndef _WIN32
-    mkdir(dirname.c_str(), 0777);
-#else
-    mkdir(dirname.c_str());
-#endif
-}
-
 void V3File::createMakeDir() {
     static bool created = false;
     if (!created) {
 	created = true;
-	createDir(v3Global.opt.makeDir());
+	V3Os::createDir(v3Global.opt.makeDir());
     }
 }
 
@@ -757,7 +750,7 @@ void V3OutFormatter::putsQuoted(const char* strg) {
     // Quote \ and " for use inside C programs
     // Don't use to quote a filename for #include - #include doesn't \ escape.
     putcNoTracking('"');
-    string quoted = AstNode::quoteName(strg);
+    string quoted = V3Number::quoteNameControls(strg);
     for (const char* cp=quoted.c_str(); *cp; cp++) {
 	putcNoTracking (*cp);
     }
