@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2014 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,6 +34,7 @@
 #include <iomanip>
 #include <vector>
 #include <list>
+#include <map>
 
 #include "V3Global.h"
 #include "V3Gate.h"
@@ -303,6 +304,7 @@ private:
     V3Double0		m_statSigs;	// Statistic tracking
     V3Double0		m_statRefs;	// Statistic tracking
     V3Double0		m_statDedupLogic;	// Statistic tracking
+    V3Double0		m_statAssignMerged;	// Statistic tracking
 
     // METHODS
     void iterateNewStmt(AstNode* nodep, const char* nonReducibleReason, const char* consumeReason) {
@@ -345,6 +347,7 @@ private:
     }
 
     void optimizeSignals(bool allowMultiIn);
+    bool elimLogicOkOutputs(GateLogicVertex* consumeVertexp, const GateOkVisitor& okVisitor);
     void optimizeElimVar(AstVarScope* varscp, AstNode* logicp, AstNode* consumerp);
     void warnSignals();
     void consumedMark();
@@ -352,6 +355,7 @@ private:
     void consumedMove();
     void replaceAssigns();
     void dedupe();
+    void mergeAssigns();
 
     // VISITORS
     virtual void visit(AstNetlist* nodep, AstNUser*) {
@@ -368,6 +372,7 @@ private:
 	optimizeSignals(true);
 	// Remove redundant logic
 	if (v3Global.opt.oDedupe()) dedupe();
+	if (v3Global.opt.oAssemble()) mergeAssigns();
 	// Warn
 	warnSignals();
 	consumedMark();
@@ -506,6 +511,7 @@ public:
 	V3Stats::addStat("Optimizations, Gate sigs deleted", m_statSigs);
 	V3Stats::addStat("Optimizations, Gate inputs replaced", m_statRefs);
 	V3Stats::addStat("Optimizations, Gate sigs deduped", m_statDedupLogic);
+	V3Stats::addStat("Optimizations, Gate assign merged", m_statAssignMerged);
     }
 };
 
@@ -587,40 +593,51 @@ void GateVisitor::optimizeSignals(bool allowMultiIn) {
 			if (debug()>=5) logicp->dumpTree(cout,"\telimVar:  ");
 			if (debug()>=5) substp->dumpTree(cout,"\t  subst:  ");
 			++m_statSigs;
-			while (V3GraphEdge* edgep = vvertexp->outBeginp()) {
+			bool removedAllUsages = true;
+			for (V3GraphEdge* edgep = vvertexp->outBeginp();
+			     edgep; ) {
 			    GateLogicVertex* consumeVertexp = dynamic_cast<GateLogicVertex*>(edgep->top());
 			    AstNode* consumerp = consumeVertexp->nodep();
-			    optimizeElimVar(vvertexp->varScp(), substp, consumerp);
-			    // If the new replacement referred to a signal,
-			    // Correct the graph to point to this new generating variable
-			    const GateVarRefList& rhsVarRefs = okVisitor.rhsVarRefs();
-			    for (GateVarRefList::const_iterator it = rhsVarRefs.begin();
-				 it != rhsVarRefs.end(); ++it) {
-				AstVarScope* newvarscp = (*it)->varScopep();
-				UINFO(9,"         Point-to-new vertex "<<newvarscp<<endl);
-				GateVarVertex* varvertexp = makeVarVertex(newvarscp);
-				new V3GraphEdge(&m_graph, varvertexp, consumeVertexp, 1);
-				// Propagate clock attribute onto generating node
-				varvertexp->propagateAttrClocksFrom(vvertexp);
+			    if (!elimLogicOkOutputs(consumeVertexp, okVisitor/*ref*/)) {
+				// Cannot optimize this replacement
+				removedAllUsages = false;
+				edgep = edgep->outNextp();
+			    } else {
+				optimizeElimVar(vvertexp->varScp(), substp, consumerp);
+				// If the new replacement referred to a signal,
+				// Correct the graph to point to this new generating variable
+				const GateVarRefList& rhsVarRefs = okVisitor.rhsVarRefs();
+				for (GateVarRefList::const_iterator it = rhsVarRefs.begin();
+				     it != rhsVarRefs.end(); ++it) {
+				    AstVarScope* newvarscp = (*it)->varScopep();
+				    UINFO(9,"         Point-to-new vertex "<<newvarscp<<endl);
+				    GateVarVertex* varvertexp = makeVarVertex(newvarscp);
+				    new V3GraphEdge(&m_graph, varvertexp, consumeVertexp, 1);
+				    // Propagate clock attribute onto generating node
+				    varvertexp->propagateAttrClocksFrom(vvertexp);
+				}
+				// Remove the edge
+				edgep->unlinkDelete(); edgep=NULL;
+				++m_statRefs;
+				edgep = vvertexp->outBeginp();
 			    }
-			    // Remove the edge
-			    edgep->unlinkDelete(); edgep=NULL;
-			    ++m_statRefs;
 			}
-			// Remove input links
-			while (V3GraphEdge* edgep = vvertexp->inBeginp()) {
-			    edgep->unlinkDelete(); edgep=NULL;
+			if (removedAllUsages) {
+			    // Remove input links
+			    while (V3GraphEdge* edgep = vvertexp->inBeginp()) {
+				edgep->unlinkDelete(); edgep=NULL;
+			    }
+			    // Clone tree so we remember it for tracing, and keep the pointer
+			    // to the "ALWAYS" part of the tree as part of this statement
+			    // That way if a later signal optimization that retained a pointer to the always
+			    // can optimize it further
+			    logicp->unlinkFrBack();
+			    vvertexp->varScp()->valuep(logicp);
+			    logicp = NULL;
+			    // Mark the vertex so we don't mark it as being unconsumed in the next step
+			    vvertexp->user(true);
+			    logicVertexp->user(true);
 			}
-			// Clone tree so we remember it for tracing, and keep the pointer
-			// to the "ALWAYS" part of the tree as part of this statement
-			// That way if a later signal optimization that retained a pointer to the always
-			// can optimize it further
-			logicp->unlinkFrBack();
-			vvertexp->varScp()->valuep(logicp);
-			logicp = NULL;
-			// Mark the vertex so we don't mark it as being unconsumed in the next step
-			vvertexp->user(true);
-			logicVertexp->user(true);
 		    }
 		}
 	    }
@@ -628,6 +645,30 @@ void GateVisitor::optimizeSignals(bool allowMultiIn) {
     }
 }
 
+bool GateVisitor::elimLogicOkOutputs(GateLogicVertex* consumeVertexp, const GateOkVisitor& okVisitor) {
+    // Return true if can optimize
+    // Return false if the consuming logic has an output signal that the replacement logic has as an input
+    typedef set<AstVarScope*> VarScopeSet;
+    // Use map to find duplicates between two lists
+    VarScopeSet varscopes;
+    // Replacement logic usually has shorter input list, so faster to build list based on it
+    const GateVarRefList& rhsVarRefs = okVisitor.rhsVarRefs();
+    for (GateVarRefList::const_iterator it = rhsVarRefs.begin();
+	 it != rhsVarRefs.end(); ++it) {
+	AstVarScope* vscp = (*it)->varScopep();
+	if (varscopes.find(vscp) == varscopes.end()) varscopes.insert(vscp);
+    }
+    for (V3GraphEdge* edgep = consumeVertexp->outBeginp(); edgep; edgep = edgep->outNextp()) {
+	GateVarVertex* consVVertexp = dynamic_cast<GateVarVertex*>(edgep->top());
+	AstVarScope* vscp = consVVertexp->varScp();
+	if (varscopes.find(vscp) != varscopes.end()) {
+	    UINFO(9,"    Block-unopt, insertion generates input vscp "<<vscp<<endl);
+	    return false;
+	}
+    }
+    return true;
+}
+
 void GateVisitor::replaceAssigns() {
     for (V3GraphVertex* itp = m_graph.verticesBeginp(); itp; itp=itp->verticesNextp()) {
 	if (GateVarVertex* vvertexp = dynamic_cast<GateVarVertex*>(itp)) {
@@ -1009,7 +1050,6 @@ private:
 		lvertexp->user(true);
 	    }
 	}
-
 	return NULL;
     }
 
@@ -1063,6 +1103,132 @@ void GateVisitor::dedupe() {
     m_statDedupLogic += deduper.numDeduped();
 }
 
+
+//######################################################################
+// Recurse through the graph, try to merge assigns 
+
+class GateMergeAssignsGraphVisitor : public GateGraphBaseVisitor {
+private:
+    // NODE STATE
+    AstNodeAssign* m_assignp;
+    AstActive*     m_activep;
+    GateLogicVertex* m_logicvp;
+    V3Graph*         m_graphp;
+    V3Double0        m_numMergedAssigns;	// Statistic tracking
+
+
+    // assemble two Sel into one if possible 
+    AstSel* merge(AstSel* pre, AstSel* cur) {
+	AstVarRef* preVarRefp = pre->fromp()->castVarRef();
+	AstVarRef* curVarRefp = cur->fromp()->castVarRef();
+	if (!preVarRefp || !curVarRefp || !curVarRefp->same(preVarRefp)) return NULL; // not the same var
+	AstConst* pstart = pre->lsbp()->castConst();
+	AstConst* pwidth = pre->widthp()->castConst();
+	AstConst* cstart = cur->lsbp()->castConst();
+	AstConst* cwidth = cur->widthp()->castConst();
+	if (!pstart || !pwidth || !cstart || !cwidth) return NULL; // too complicated
+	if (cur->lsbConst()+cur->widthConst() == pre->lsbConst())
+	    return new AstSel(curVarRefp->fileline(), curVarRefp->cloneTree(false), cur->lsbConst(), pre->widthConst()+cur->widthConst());
+	else return NULL;
+    }
+
+    virtual AstNUser* visit(GateVarVertex *vvertexp, AstNUser*) {
+	for (V3GraphEdge* edgep = vvertexp->inBeginp(); edgep; ) {
+	    V3GraphEdge* oldedgep = edgep;
+	    edgep = edgep->inNextp();  // for recursive since the edge could be deleted
+	    if (GateLogicVertex* lvertexp = dynamic_cast<GateLogicVertex*>(oldedgep->fromp())) {
+		if (AstNodeAssign* assignp = lvertexp->nodep()->castNodeAssign()) {
+		    //if (lvertexp->outSize1() && assignp->lhsp()->castSel()) {
+		    if (assignp->lhsp()->castSel() && lvertexp->outSize1()) {
+			UINFO(9, "assing to the nodep["<<assignp->lhsp()->castSel()->lsbConst()<<"]"<<endl);
+			// first assign with Sel-lhs
+			if (!m_activep) m_activep = lvertexp->activep();
+			if (!m_logicvp) m_logicvp = lvertexp;
+			if (!m_assignp) m_assignp = assignp;
+
+			// not under the same active
+			if (m_activep != lvertexp->activep()) {
+			    m_activep = lvertexp->activep();
+			    m_logicvp = lvertexp;
+			    m_assignp = assignp;
+			    continue;
+			}
+
+			AstSel* preselp = m_assignp->lhsp()->castSel();
+			AstSel* curselp = assignp->lhsp()->castSel();
+			if (!preselp || !curselp) continue;
+
+			if (AstSel* newselp = merge(preselp, curselp)) {
+			    UINFO(5, "assemble to new sel: "<<newselp<<endl);
+			    // replace preSel with newSel
+			    preselp->replaceWith(newselp); preselp->deleteTree(); preselp = NULL;
+			    // create new rhs for pre assignment
+			    AstNode* newrhsp = new AstConcat(m_assignp->rhsp()->fileline(), m_assignp->rhsp()->cloneTree(false), assignp->rhsp()->cloneTree(false));
+			    AstNode* oldrhsp = m_assignp->rhsp();
+			    oldrhsp->replaceWith(newrhsp); oldrhsp->deleteTree(); oldrhsp = NULL;
+			    m_assignp->dtypeChgWidthSigned(m_assignp->width()+assignp->width(), m_assignp->width()+assignp->width(), AstNumeric::fromBool(true));
+			    // don't need to delete, will be handled
+			    //assignp->unlinkFrBack(); assignp->deleteTree(); assignp = NULL;
+
+			    // update the graph
+			    {
+				// delete all inedges to lvertexp
+				if (!lvertexp->inEmpty()) { 
+				    for (V3GraphEdge* ledgep = lvertexp->inBeginp(); ledgep; ) {
+					V3GraphEdge* oedgep = ledgep;
+					ledgep = ledgep->inNextp();
+					GateEitherVertex* fromvp = dynamic_cast<GateEitherVertex*>(oedgep->fromp());
+					new V3GraphEdge(m_graphp, fromvp, m_logicvp, 1);
+					oedgep->unlinkDelete(); oedgep = NULL;
+				    }
+				}
+				// delete all outedges to lvertexp, only one
+				oldedgep->unlinkDelete(); oldedgep = NULL;
+			    }
+			    ++m_numMergedAssigns;
+			} else {
+			    m_assignp = assignp;
+			    m_logicvp = lvertexp;
+			}
+		    }
+		}
+	    }
+	}
+	return NULL;
+    }
+
+    virtual AstNUser* visit(GateLogicVertex* lvertexp, AstNUser* vup) {
+	return NULL;
+    }
+
+public:
+    GateMergeAssignsGraphVisitor(V3Graph* graphp) {
+	m_assignp = NULL;
+	m_activep = NULL;
+	m_logicvp = NULL;
+	m_numMergedAssigns = 0;
+	m_graphp = graphp;
+    }
+    void mergeAssignsTree(GateVarVertex* vvertexp) {
+	vvertexp->accept(*this);
+    }
+    V3Double0 numMergedAssigns() { return m_numMergedAssigns; }
+};
+
+
+//----------------------------------------------------------------------
+
+void GateVisitor::mergeAssigns() {
+    GateMergeAssignsGraphVisitor merger(&m_graph);
+    for (V3GraphVertex* itp = m_graph.verticesBeginp(); itp; itp=itp->verticesNextp()) {
+	if (GateVarVertex* vvertexp = dynamic_cast<GateVarVertex*>(itp)) {
+	    merger.mergeAssignsTree(vvertexp);
+	}
+    }
+    m_statAssignMerged += merger.numMergedAssigns();
+}
+
+
 //######################################################################
 // Convert VARSCOPE(ASSIGN(default, VARREF)) to just VARSCOPE(default)
 
@@ -1100,4 +1266,5 @@ void V3Gate::gateAll(AstNetlist* nodep) {
     UINFO(2,__FUNCTION__<<": "<<endl);
     GateVisitor visitor (nodep);
     GateDeassignVisitor deassign (nodep);
+    V3Global::dumpCheckGlobalTree("gate.tree", 0, v3Global.opt.dumpTreeLevel(__FILE__) >= 3);
 }
