@@ -52,6 +52,7 @@ my $opt_gdbsim;
 my $opt_ghdl;
 my $opt_iv;
 my $opt_jobs = 1;
+my $opt_ms;
 my $opt_nc;
 my $opt_optimize;
 my $opt_site;
@@ -79,6 +80,7 @@ if (! GetOptions (
 		  "help"	=> \&usage,
 		  "iverilog!"	=> \$opt_iv,
 		  "j=i"		=> \$opt_jobs,
+		  "ms!"		=> \$opt_ms,
 		  "nc!"		=> \$opt_nc,
 		  "optimize:s"	=> \$opt_optimize,
 		  "site!"	=> \$opt_site,
@@ -102,7 +104,7 @@ $opt_jobs = calc_jobs() if defined $opt_jobs && $opt_jobs==0;
 
 $Fork->max_proc($opt_jobs);
 
-if (!$opt_atsim && !$opt_ghdl && !$opt_iv && !$opt_vcs && !$opt_nc && !$opt_vlt) {
+if (!$opt_atsim && !$opt_ghdl && !$opt_iv && !$opt_vcs && !$opt_ms && !$opt_nc && !$opt_vlt) {
     $opt_vlt = 1;
 }
 
@@ -115,9 +117,15 @@ if ($#opt_tests<0) {
     foreach my $dir (@Test_Dirs) {
 	my @stats = stat($dir);  # Uniquify by inode, so different paths to same place get combined
 	next if !$stats[1] || $uniq{$stats[1]}++;
-	push @opt_tests, glob ("${dir}/t_*.pl");
+	push @opt_tests, sort(glob ("${dir}/t_*.pl"));
     }
 }
+if ($#opt_tests>=2 && $opt_jobs>=2) {
+    # Without this tests such as t_debug_sigsegv_bt_bad.pl will occasionally
+    # block on input and cause a SIGSTOP, then a "fg" was needed to resume testing.
+    print STDERR "== Many jobs; redirecting STDIN\n";
+    open(STDIN,  "+>/dev/null");
+}
 
 mkdir "obj_dir";
 
@@ -129,6 +137,7 @@ foreach my $testpl (@opt_tests) {
     one_test(pl_filename => $testpl, atsim=>1) if $opt_atsim;
     one_test(pl_filename => $testpl, ghdl=>1) if $opt_ghdl;
     one_test(pl_filename => $testpl, iv=>1) if $opt_iv;
+    one_test(pl_filename => $testpl, ms=>1) if $opt_ms;
     one_test(pl_filename => $testpl, nc=>1) if $opt_nc;
     one_test(pl_filename => $testpl, vcs=>1) if $opt_vcs;
     one_test(pl_filename => $testpl, vlt=>1, 'v3'=>1) if $opt_vlt;
@@ -295,6 +304,7 @@ sub new {
     $self->{mode} ||= "vcs" if $self->{vcs};
     $self->{mode} ||= "vlt" if $self->{vlt};
     $self->{mode} ||= "nc" if $self->{nc};
+    $self->{mode} ||= "ms" if $self->{ms};
     $self->{mode} ||= "iv" if $self->{iv};
 
     # For backward compatibility, the verilator tests have no prefix
@@ -370,6 +380,11 @@ sub new {
 				  .($opt_trace ? " +access+r":"")))],
 	nc_flags2 => [],  # Overridden in some sim files
 	nc_run_flags => [split(/\s+/,"+licqueue -q +assert +sv -R")],
+	# ModelSim
+	ms => 0,
+	ms_flags => [split(/\s+/,("-sv -work $self->{obj_dir}/work"))],
+	ms_flags2 => [],  # Overridden in some sim files
+	ms_run_flags => [split(/\s+/,"-lib $self->{obj_dir}/work -c -do 'run -all;quit' ")],
 	# Verilator
 	vlt => 0,
 	'v3' => 0,
@@ -378,6 +393,7 @@ sub new {
 			    "-OD",  # As currently disabled unless -O3
 			    "--debug-check"],
 	verilator_flags2 => [],
+	verilator_flags3 => ["--clk clk"],
 	verilator_make_gcc => 1,
 	verilated_debug => $Opt_Verilated_Debug,
 	stdout_filename => undef,	# Redirect stdout
@@ -388,7 +404,7 @@ sub new {
     $self->{stats} ||= "$self->{obj_dir}/V".$self->{name}."__stats.txt";
     $self->{status_filename} ||= "$self->{obj_dir}/V".$self->{name}.".status";
     $self->{run_log_filename} ||= "$self->{obj_dir}/vlt_sim.log";
-    $self->{coverage_filename} ||= "$self->{obj_dir}/vlt_coverage.pl";
+    $self->{coverage_filename} ||= "$self->{obj_dir}/coverage.dat";
     $self->{vcd_filename}  ||= "$self->{obj_dir}/sim.vcd";
     $self->{main_filename} ||= "$self->{obj_dir}/$self->{VM_PREFIX}__main.cpp";
     ($self->{top_filename} = $self->{pl_filename}) =~ s/\.pl$//;
@@ -487,7 +503,8 @@ sub compile_vlt_flags {
     my $checkflags = join(' ',@{$param{v_flags}},
 			  @{$param{v_flags2}},
 			  @{$param{verilator_flags}},
-			  @{$param{verilator_flags2}});
+			  @{$param{verilator_flags2}},
+			  @{$param{verilator_flags3}});
     $self->{sc} = 1 if ($checkflags =~ /-sc\b/);
     $self->{sp} = 1 if ($checkflags =~ /-sp\b/);
     $self->{trace} = 1 if ($opt_trace || $checkflags =~ /-trace\b/);
@@ -518,6 +535,7 @@ sub compile_vlt_flags {
 		   "--prefix ".$param{VM_PREFIX},
 		   @verilator_flags,
 		   @{$param{verilator_flags2}},
+		   @{$param{verilator_flags3}},
 		   @{$param{v_flags}},
 		   @{$param{v_flags2}},
 		   $param{top_filename},
@@ -610,6 +628,21 @@ sub compile {
 			  @more_args
 			  ]);
     }
+    elsif ($param{ms}) {
+	$self->_make_top();
+	$self->_run(logfile=>"$self->{obj_dir}/ms_compile.log",
+		    fails=>$param{fails},
+		    cmd=>[("vlib $self->{obj_dir}/work && "),
+                          ($ENV{VERILATOR_MODELSIM}||"vlog"),
+			  @{$param{ms_flags}},
+			  @{$param{ms_flags2}},
+			  @{$param{v_flags}},
+			  @{$param{v_flags2}},
+			  $param{top_filename},
+			  $param{top_shell_filename},
+			  @{$param{v_other_filenames}}
+			  ]);
+    }
     elsif ($param{iv}) {
 	$self->_make_top();
 	my @cmd = (($ENV{VERILATOR_IVERILOG}||"iverilog"),
@@ -633,10 +666,6 @@ sub compile {
 	    $self->skip("Test requires SystemC; ignore error since not installed\n");
 	    return 1;
 	}
-	elsif ($self->{coverage} && !$Have_System_Perl) {
-	    $self->skip("Test requires SystemPerl; ignore error since not installed\n");
-	    return 1;
-	}
 
 	if (!$param{fails} && $param{verilator_make_gcc}
 	    && $param{make_main}) {
@@ -730,6 +759,18 @@ sub execute {
 		    expect=>$param{iv_run_expect},	# non-verilator expect isn't the same
 		    );
     }
+    elsif ($param{ms}) {
+	$self->_run(logfile=>"$self->{obj_dir}/ms_sim.log",
+		    fails=>$param{fails},
+		    cmd=>["echo q | ".$run_env.($ENV{VERILATOR_MODELSIM}||"vsim"),
+			  @{$param{ms_run_flags}},
+			  @{$param{all_run_flags}},
+                          (" top")
+		          ],
+		    %param,
+		    expect=>$param{ms_run_expect},	# non-verilator expect isn't the same
+		    );
+    }
     elsif ($param{nc}) {
 	$self->_run(logfile=>"$self->{obj_dir}/nc_sim.log",
 		    fails=>$param{fails},
@@ -1143,7 +1184,7 @@ sub _make_main {
 
     if ($self->{coverage}) {
 	$fh->print("#if VM_COVERAGE\n");
-	$fh->print("    SpCoverage::write(\"",$self->{coverage_filename},"\");\n");
+	$fh->print("    VerilatedCov::write(\"",$self->{coverage_filename},"\");\n");
 	$fh->print("#endif //VM_COVERAGE\n");
     }
     if ($self->{trace}) {
@@ -1659,6 +1700,15 @@ Set to 0 to disable the automatic creation of a top level shell to run the
 executable (for example when a hand-written test wrapper is provided using
 C<--exe>).
 
+=item ms_flags
+
+=item ms_flags2
+
+=item ms_run_flags
+
+The equivalent of C<v_flags>, C<v_flags2> and C<all_run_flags>, but only
+for use with the ModelSim simulator.
+
 =item nc_flags
 
 =item nc_flags2
@@ -1689,7 +1739,8 @@ A list of standard Verilog simulator flags to be passed to the simulator
 compiler (Verilator or one of the other simulators). Unlike v_flags, these
 options may be overridden in some simulation files.
 
-Similar sets of flags exist for atsim, GHDL, Cadence NC and Synopsys VCS.
+Similar sets of flags exist for atsim, GHDL, Cadence NC, Modelsim and
+Synopsys VCS.
 
 =item vcs_flags
 
@@ -1840,6 +1891,10 @@ Run using Icarus Verilog simulator.
 Run number of parallel tests, or 0 to determine the count based on the
 number of cores installed.  Requires Perl's Parallel::Forker package.
 
+=item --ms
+
+Run using ModelSim simulator.
+
 =item --nc
 
 Run using Cadence NC-Verilog simulator.
@@ -1900,6 +1955,10 @@ Command to use to invoke GHDL.
 
 Command to use to invoke Icarus Verilog.
 
+=item VERILATOR_MODELSIM
+
+Command to use to invoke ModelSim.
+
 =item VERILATOR_NCVERILOG
 
 Command to use to invoke ncverilog.
@@ -1918,7 +1977,7 @@ Command to use to invoke VCS.
 
 The latest version is available from L<http://www.veripool.org/>.
 
-Copyright 2003-2014 by Wilson Snyder.  Verilator is free software; you can
+Copyright 2003-2015 by Wilson Snyder.  Verilator is free software; you can
 redistribute it and/or modify it under the terms of either the GNU Lesser
 General Public License Version 3 or the Perl Artistic License Version 2.0.
 
