@@ -11,7 +11,7 @@ compile (
 	 verilator_flags2 => ["--lint-only"],
 	 fails=>1,
 	 expect=>
-q{%Error: t/t_inst_misarray_bad.v:\d+: Illegal input port connection 'foo', mismatch between port which is not an array, and expression which is an array.
+q{%Error: t/t_inst_misarray_bad.v:\d+: Illegal assignment of constant to unpacked array
 %Error: Exiting due to.*},
 	 );
 
